package org.fossasia.susi.ai.helper;

import android.content.Context;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;

import com.google.gson.Gson;

import org.fossasia.susi.ai.MainApplication;
import org.fossasia.susi.ai.R;
import org.fossasia.susi.ai.rest.clients.BaseUrl;
import org.fossasia.susi.ai.rest.responses.susi.SusiBaseUrls;

import java.util.Set;

/**
 * <h1>Helper class to store preferences of user.</h1>
 *
 * @author Rajan Maurya
 */
public class PrefManager {

    private static final String SUSI_BASE_URLS = "preferences_base_urls";
    private static final String SUSI_RUNNING_BASE_URL = "preferences_running_base_url";

    private static Gson gson = new Gson();

    private static SharedPreferences getPreferences() {
        return PreferenceManager.getDefaultSharedPreferences(getContext());
    }

    /**
     * Clear prefs.
     */
    public static void clearPrefs() {
        getPreferences().edit().clear().apply();
    }


    /**
     * Gets int.
     *
     * @param preferenceKey          the preference key
     * @param preferenceDefaultValue the preference default value
     * @return the int
     */
    public static int getInt(String preferenceKey, int preferenceDefaultValue) {
        return getPreferences().getInt(preferenceKey, preferenceDefaultValue);
    }

    /**
     * Put int.
     *
     * @param preferenceKey   the preference key
     * @param preferenceValue the preference value
     */
    public static void putInt(String preferenceKey, int preferenceValue) {
        getPreferences().edit().putInt(preferenceKey, preferenceValue).apply();
    }

    /**
     * Gets long.
     *
     * @param preferenceKey          the preference key
     * @param preferenceDefaultValue the preference default value
     * @return the long
     */
    public static long getLong(String preferenceKey, long preferenceDefaultValue) {
        return getPreferences().getLong(preferenceKey, preferenceDefaultValue);
    }

    /**
     * Put long.
     *
     * @param preferenceKey   the preference key
     * @param preferenceValue the preference value
     */
    public static void putLong(String preferenceKey, long preferenceValue) {
        getPreferences().edit().putLong(preferenceKey, preferenceValue).apply();
    }

    /**
     * Gets float.
     *
     * @param preferenceKey          the preference key
     * @param preferenceDefaultValue the preference default value
     * @return the float
     */
    public static float getFloat(String preferenceKey, float preferenceDefaultValue) {
        return getPreferences().getFloat(preferenceKey, preferenceDefaultValue);
    }

    /**
     * Put float.
     *
     * @param preferenceKey   the preference key
     * @param preferenceValue the preference value
     */
    public static void putFloat(String preferenceKey, float preferenceValue) {
        getPreferences().edit().putFloat(preferenceKey, preferenceValue).apply();
    }

    /**
     * Gets boolean.
     *
     * @param preferenceKey          the preference key
     * @param preferenceDefaultValue the preference default value
     * @return the boolean
     */
    public static boolean getBoolean(int preferenceKey, boolean preferenceDefaultValue) {
        return getPreferences().getBoolean(getContext().getString(preferenceKey), preferenceDefaultValue);
    }

    /**
     * Put boolean.
     *
     * @param preferenceKey   the preference key
     * @param preferenceValue the preference value
     */
    public static void putBoolean(int preferenceKey, boolean preferenceValue) {
        getPreferences().edit().putBoolean(getContext().getString(preferenceKey), preferenceValue).apply();
    }

    /**
     * Gets string.
     *
     * @param preferenceKey          the preference key
     * @param preferenceDefaultValue the preference default value
     * @return the string
     */
    public static String getString(String preferenceKey, String preferenceDefaultValue) {
        return getPreferences().getString(preferenceKey, preferenceDefaultValue);
    }

    /**
     * Put string.
     *
     * @param preferenceKey   the preference key
     * @param preferenceValue the preference value
     */
    public static void putString(String preferenceKey, String preferenceValue) {
        getPreferences().edit().putString(preferenceKey, preferenceValue).apply();
    }

    /**
     * Put string set.
     *
     * @param preferencesKey the preferences key
     * @param values         the values
     */
    public static void putStringSet(String preferencesKey, Set<String> values) {
        getPreferences().edit().putStringSet(preferencesKey, values).apply();
    }

    /**
     * Gets string set.
     *
     * @param preferencesKey the preferences key
     * @return the string set
     */
    public static Set<String> getStringSet(String preferencesKey) {
        return getPreferences().getStringSet(preferencesKey, null);
    }

    /**
     * This method for retrieving current Susi base url from SharedPreferences
     *
     * @return String Running Susi Base Url
     */
    public static String getSusiRunningBaseUrl() {

        if (getBoolean(R.string.susi_server_selected_key, true)) {
            return getString(SUSI_RUNNING_BASE_URL, BaseUrl.SUSI_DEFAULT_BASE_URL);
        }
        return getString("custom_server", "null");
    }

    /**
     * This method will give current susi running base url.
     *
     * @param runningBaseUrl Running Base ur;
     */
    public static void setSusiRunningBaseUrl(String runningBaseUrl) {
        getPreferences().edit().putString(SUSI_RUNNING_BASE_URL, runningBaseUrl).apply();
    }

    /**
     * This Method will save the Susi base urls that will be fetched from server.
     *
     * @param susiBaseUrls Susi All base urls
     */
    public static void saveBaseUrls(SusiBaseUrls susiBaseUrls) {
        putString(SUSI_BASE_URLS, gson.toJson(susiBaseUrls));
    }

    /**
     * This Method for retrieving All Susi base url from SharedPreferences
     *
     * @return SusiBaseUrls base urls
     */
    public static SusiBaseUrls getBaseUrls() {
        return gson.fromJson(getString(SUSI_BASE_URLS, "null"), SusiBaseUrls.class);
    }


    /**
     * Has token expired boolean.
     *
     * @return the boolean
     */
    public static boolean hasTokenExpired() {
        long validTime = getLong(Constant.TOKEN_VALIDITY, 0);
        return validTime < System.currentTimeMillis();
    }

    /**
     * Gets token.
     *
     * @return the token
     */
    public static String getToken() {
        return hasTokenExpired() ? null : getString(Constant.ACCESS_TOKEN, null);
    }

    /**
     * Clear token.
     */
    public static void clearToken() {
        SharedPreferences.Editor editor = getPreferences().edit();
        editor.remove(Constant.ACCESS_TOKEN);
        editor.remove(Constant.TOKEN_VALIDITY);
        editor.apply();
    }

    /**
     * Check speech output pref boolean.
     *
     * @return the boolean
     */
    public static boolean checkSpeechOutputPref() {
        return PrefManager.getBoolean(R.string.settings_speechPreference_key, true);
    }

    /**
     * Check speech always pref boolean.
     *
     * @return the boolean
     */
    public static boolean checkSpeechAlwaysPref() {
        return PrefManager.getBoolean(R.string.settings_speechAlways_key, false);
    }

    /**
     * Check for hotword detection pref boolean
     *
     * @return the boolean
     */
    public static boolean checkHotwordPref() {
        return PrefManager.getBoolean(R.string.hotword_detection_key, false);
    }

    /**
     * Check mic input boolean.
     *
     * @return the boolean
     */
    public static boolean checkMicInput(Context context) {
        return MediaUtil.INSTANCE.isAvailableForVoiceInput(context);
    }

    private static Context getContext() {
        return MainApplication.getInstance().getApplicationContext();
    }
}


